"""
A Django template tag library which allows to download external images, store
them locally and return the local path to locally stored image to a desired
context variable, along with `width` and `height` of the image fetched. Caches
the fetched images locally for the given time (set in settings). You could,
for example, use this app to solve the problems with displaying of a mixed
content (assets loaded from HTTP and HTTPS sources).

- default_app_config: Default Django app config.
"""

from .utils import obtain_image

__title__ = 'eximagination'
__version__ = '0.8.3'
__build__ = 0x00000b
__author__ = 'Artur Barseghyan <artur.barseghyan@gmail.com>'
__copyright__ = '2013-2017 Artur Barseghyan'
__license__ = 'GPL 2.0/LGPL 2.1'
__all__ = (
    'obtain_image',
    'default_app_config',
)

default_app_config = 'eximagination.apps.Config'



from __future__ import print_function

import os
import unittest

from . import obtain_image
from .helpers import project_dir

__title__ = 'eximagination.tests'
__author__ = 'Artur Barseghyan <artur.barseghyan@gmail.com>'
__copyright__ = '2013-2017 Artur Barseghyan'
__license__ = 'GPL 2.0/LGPL 2.1'

LOG_INFO = True


def log_info(func):
    """Log some useful info."""
    if not LOG_INFO:
        return func

    def inner(self, *args, **kwargs):
        result = func(self, *args, **kwargs)

        print('\n\n%s' % func.__name__)
        print('============================')
        if func.__doc__:
            print('""" %s """' % func.__doc__.strip())
        print('----------------------------')
        if result is not None:
            print(result)
        print('\n++++++++++++++++++++++++++++')

        return result
    return inner


class EximaginationUtilsTest(unittest.TestCase):
    """Tests of ``eximagination.utils.obtain_image`` function."""

    def setUp(self):
        """Set up."""
        self.image_url = 'http://www.google.com/intl/en/images/logo.gif'
        self.force_update = False

        # Where eximagination cached images will be stored
        self.media_root = project_dir('tmp')
        self.media_url = '/tmp/'  # Media URL for stored images
        self.media_relative_root = 'tmp/'  # Relative root for images
        # After 30 days we re-fetch the file anyway.
        self.expiration_interval = 2592000
        self.debug = True

        try:
            os.stat(self.media_root)
        except Exception:
            os.mkdir(self.media_root)

    @log_info
    def test_01_obtain_image(self):
        """Test obtain image."""
        file_data = obtain_image(
            image_source=self.image_url,
            save_to=self.media_root,
            media_url=self.media_relative_root,
            force_update=self.force_update,
            expiration_interval=self.expiration_interval,
            debug=self.debug
        )

        try:
            filename = file_data[0]
            ei_width = file_data[1]  # Original width of the obtained image
            ei_height = file_data[2]  # Original height of the obtained image
        except Exception as err:
            print(file_data)
            print(err)
            filename = None
            ei_width = None
            ei_height = None

        res = {
            'filename': filename,
            'ei_width': ei_width,
            'ei_height': ei_height
        }

        self.assertTrue(filename is not None)
        self.assertTrue(ei_width is not None)
        self.assertTrue(ei_height is not None)

        return res


# Skipping from non-Django tests.
if os.environ.get("DJANGO_SETTINGS_MODULE", None):
    from django.conf import settings
    from django.test import LiveServerTestCase
    from django.core.management import call_command

    from selenium import webdriver
    from selenium.webdriver.firefox.firefox_binary import FirefoxBinary

    class EximaginationTemplatetagsTest(LiveServerTestCase):
        """Tests of ``exinagination.templatetags.eximaginate`` module."""

        @classmethod
        def setUpClass(cls):
            """Set up."""
            firefox_bin_path = getattr(settings, 'FIREFOX_BIN_PATH', None)
            phantom_js_executable_path = getattr(
                settings, 'PHANTOM_JS_EXECUTABLE_PATH', None
            )
            if phantom_js_executable_path is not None:
                if phantom_js_executable_path:
                    cls.selenium = webdriver.PhantomJS(
                        executable_path=phantom_js_executable_path
                    )
                else:
                    cls.selenium = webdriver.PhantomJS()
            elif firefox_bin_path:
                binary = FirefoxBinary(firefox_bin_path)
                cls.selenium = webdriver.Firefox(firefox_binary=binary)
            else:
                cls.selenium = webdriver.Firefox()
            super(EximaginationTemplatetagsTest, cls).setUpClass()

        @classmethod
        def tearDownClass(cls):
            """Tear down."""
            try:
                cls.selenium.quit()
            except Exception as err:
                print(err)
            super(EximaginationTemplatetagsTest, cls).tearDownClass()
            call_command('flush', verbosity=0, interactive=False,
                         reset_sequences=False,
                         allow_cascade=False,
                         inhibit_post_migrate=False)

        @log_info
        def test_templatetags(self):
            """Test template tags."""
            workflow = []

            self.selenium.get(self.live_server_url)

            img1 = self.selenium.find_element_by_id("image1")
            self.assertTrue(img1.get_attribute('src').endswith('gif'))
            workflow.append(img1.get_attribute('src'))

            img2 = self.selenium.find_element_by_id("image2")
            self.assertTrue(img2.get_attribute('src').endswith('gif'))
            workflow.append(img2.get_attribute('src'))

            return workflow


if __name__ == "__main__":
    # Tests
    unittest.main()



from __future__ import absolute_import, unicode_literals
import os
import hashlib
import glob
import time

from six.moves.urllib.request import build_opener
from six import BytesIO as StringIO

try:
    from PIL import Image
except ImportError:
    import Image

__title__ = 'eximagination.utils'
__author__ = 'Artur Barseghyan <artur.barseghyan@gmail.com>'
__copyright__ = '2013-2017 Artur Barseghyan'
__license__ = 'GPL 2.0/LGPL 2.1'
__all__ = (
    'get_new_image_filename',
    'obtain_image',
)


def get_new_image_filename(image_source):
    """Get new image filename."""
    return hashlib.md5(image_source).hexdigest()


def obtain_image(image_source='', save_to='', media_url='',
                 force_update=False, expiration_interval=None, debug=False):
    """Get an image from absolute url and saves it locally.

    Check whether image already exists in local directory and only if not -
    try to download it and save it. Validate validity of the image (rely on
    PIL Image class validation).

    :param str image_source:
    :param str save_to:
    :param str media_url:
    :param bool force_update:
    :param int expiration_interval: Expiration interval in seconds.
    :param bool debug:
    :return list: (relative_url_of_the_image,
                   original_image_width,
                   original_image_height)
    """
    # First we check if any image with such name (without extention) exists
    # in our local external images directory.
    expired = False
    try:
        filename = os.path.basename(
            glob.glob(
                os.path.join(
                    save_to,
                    get_new_image_filename(image_source)
                ) + '.*'
            )[0]
        )

        # Expiration interval check.
        full_path = os.path.join(save_to, filename)
        last_modified = os.path.getmtime(full_path)
        if time.time() - last_modified > expiration_interval:
            expired = True

        if not expired:
            # Feed the image to PIL to get width and height
            img = Image.open(full_path)
            return os.path.join(media_url, filename), img.size[0], img.size[1]
    except Exception:
        pass

    # If nothing was found in local directory (or perhaps file expired) - we
    # try to load it and save it.
    try:
        # Loading the file of unknown type into memory. We don't save the
        # image locally before it's validated.
        opener = build_opener()
        page = opener.open(image_source)
        external_image = page.read()

        # Loading into PIL image in order to check if image is a proper image
        # as well as to detect its' extension. This is probably the best way
        # to validate the image and get its' extension, since PIL Image throws
        # an exception when trying to load not a proper image.
        img = Image.open(StringIO(external_image))

        # This is our filename
        filename = hashlib.md5(
            image_source.encode('utf-8')
        ).hexdigest() + '.' + img.format.lower()

        # If filename already exists - returning the filename
        filename_full_path = os.path.join(save_to, filename)
        if os.path.exists(filename_full_path) \
                and os.path.isfile(filename_full_path) \
                and not (force_update or expired):
            return os.path.join(media_url, filename), img.size[0], img.size[1]

        # Open file for binary write and save the image.
        fout = open(filename_full_path, "wb")
        fout.write(external_image)
        fout.close()
        return os.path.join(media_url, filename), img.size[0], img.size[1]
    except Exception:
        if debug:
            raise Exception("Wrong image type or can load the image")
        else:
            return ''



__title__ = 'eximagination.exceptions'
__author__ = 'Artur Barseghyan <artur.barseghyan@gmail.com>'
__copyright__ = '2013-2017 Artur Barseghyan'
__license__ = 'GPL 2.0/LGPL 2.1'
__all__ = (
    'EximaginationException',
    'WrongImageTypeOrCantLoadTheImage',
)


class EximaginationException(Exception):
    """Base exception."""


class WrongImageTypeOrCantLoadTheImage(EximaginationException):
    """Wrong image type or can load the image exception."""




from django import template

from ..utils import obtain_image
from ..settings import (
    MEDIA_ROOT,
    MEDIA_RELATIVE_ROOT,
    EXPIRATION_INTERVAL,
    DEBUG
)

__title__ = 'eximagination.templatetags.eximaginate'
__author__ = 'Artur Barseghyan <artur.barseghyan@gmail.com>'
__copyright__ = '2013-2017 Artur Barseghyan'
__license__ = 'GPL 2.0/LGPL 2.1'
__all__ = ('eximaginate',)

register = template.Library()


class EximaginateNode(template.Node):
    """Node for ``eximaginate`` tag.

    .. code-block: html

        {% eximaginate photo as image %}
        {{ image }}

    Eximagination will automatically add ``ei_width`` and ``ei_height``
    variables to the content - those are the original width and height values
    of the obtained image.
    """
    def __init__(self, source_var, force_update, context_name=None):
        self.source_var = template.Variable(source_var)
        self.force_update = bool(force_update)
        self.context_name = context_name

    def render(self, context):
        # Note that this isn't a global constant because we need to change the
        # value for tests.
        try:
            file_data = obtain_image(
                image_source=self.source_var.resolve(context),
                save_to=MEDIA_ROOT,
                media_url=MEDIA_RELATIVE_ROOT,
                force_update=self.force_update,
                expiration_interval=EXPIRATION_INTERVAL,
                debug=DEBUG
            )
            filename = file_data[0]
            # Original width of the obtained image
            context['ei_width'] = file_data[1]
            # Original height of the obtained image
            context['ei_height'] = file_data[2]
            if self.context_name is None:
                return filename

            # We need to get here so we don't have old values in the context
            # variable.
            context[self.context_name] = filename
        except Exception:
            pass

        return ''


@register.tag
def eximaginate(parser, token):
    """Copy an external image locally.

    To just output the absolute url to the thumbnail::

        {% load eximaginate %}

        <img src="{{ MEDIA_URL }}
           {% eximaginate 'http://www.google.com/intl/en/images/logo.gif' %}">

    To put the the downloaded image URL on the context instead of just
    rendering the relative url, finish the tag with ``as [context_var_name]``:

        {% load eximaginate thumbnail %}

        {% eximaginate 'http://www.google.com/intl/en/images/logo.gif'
           as original %}

        <img src="{% thumbnail original 100x100 %}">

    Eximagination will automatically add "ei_width" and "ei_height" variables
    to the content - those are the original width and height values of the
    obtained image.

    TODO: Implement force_update feature (some extra tokens parsing)
    """
    arguments = token.split_contents()
    # Check to see if we're setting to a context variable.

    if len(arguments) < 1:
        raise template.TemplateSyntaxError(
            "{0} tag requires at least one "
            "argument".format(token.contents.split()[0])
        )

    # Get arguments
    if len(arguments) == 4:
        source_var = arguments[1]
        force_update = False  # At the moment force_update is not implemented
        context_name = arguments[3]
    elif len(arguments) == 2:
        source_var = arguments[1]
        force_update = False  # At the moment force_update is not implemented
        context_name = None
    else:
        raise template.TemplateSyntaxError(
            "{0} wrong number arguments".format(token.contents.split()[0])
        )

    # If the size argument was a correct static format, wrap it in quotes so
    # that it is compiled correctly.
    return EximaginateNode(source_var, force_update, context_name)



from .conf import get_setting

__title__ = 'eximagination.settings'
__author__ = 'Artur Barseghyan <artur.barseghyan@gmail.com>'
__copyright__ = '2013-2017 Artur Barseghyan'
__license__ = 'GPL 2.0/LGPL 2.1'
__all__ = (
    'MEDIA_ROOT',
    'MEDIA_URL',
    'MEDIA_RELATIVE_ROOT',
    'EXPIRATION_INTERVAL',
    'DEBUG'
)

MEDIA_ROOT = get_setting('MEDIA_ROOT')
MEDIA_URL = get_setting('MEDIA_URL')
MEDIA_RELATIVE_ROOT = get_setting('MEDIA_RELATIVE_ROOT')
EXPIRATION_INTERVAL = get_setting('EXPIRATION_INTERVAL')
DEBUG = get_setting('DEBUG')  # Set back to settings from DEBUG.



from django.conf import settings

from . import defaults

__title__ = 'eximagination.conf'
__author__ = 'Artur Barseghyan <artur.barseghyan@gmail.com>'
__copyright__ = '2013-2017 Artur Barseghyan'
__license__ = 'GPL 2.0/LGPL 2.1'
__all__ = ('get_setting',)


def get_setting(setting, override=None):
    """Get setting.

    Get a setting from ``eximagination`` conf module, falling back to
    the default.

    If override is not None, it will be used instead of the setting.

    :param setting: String with setting name
    :param override: Value to use when no setting is available. Defaults to
        None.
    :return: Setting value.
    """
    attr_name = 'EXIMAGINATION_{0}'.format(setting)
    if hasattr(settings, attr_name):
        return getattr(settings, attr_name)
    else:
        if hasattr(defaults, setting):
            return getattr(defaults, setting)
        else:
            return override



import os
from setuptools import setup, find_packages

try:
    readme = open(os.path.join(os.path.dirname(__file__), 'README.rst')).read()
except:
    readme = ''

version = '0.8.3'

setup(
    name='django-eximagination',
    version=version,
    description="A Django template tag library which allows to download "
                "external images, store them locally and return the local "
                "path to locally stored image to a desired context variable",
    long_description=readme,
    classifiers=[
        "Framework :: Django",
        "Programming Language :: Python",
        "Programming Language :: Python :: 2.6",
        "Programming Language :: Python :: 2.7",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.4",
        "Programming Language :: Python :: 3.5",
        "Programming Language :: Python :: 3.6",
        "Development Status :: 4 - Beta",
        "Environment :: Web Environment",
        "Intended Audience :: Developers",
        "Operating System :: OS Independent",
    ],
    keywords='eximagination, django, external images, app, python',
    author='Artur Barseghyan',
    author_email='artur.barseghyan@gmail.com',
    url='https://github.com/barseghyanartur/django-eximagination',
    package_dir={'': 'src'},
    packages=find_packages(where='./src'),
    license='GPL 2.0/LGPL 2.1',
    install_requires=[
        'six',
        'Pillow'
    ]
)



from .helpers import project_dir

__title__ = 'eximagination.defaults'
__author__ = 'Artur Barseghyan <artur.barseghyan@gmail.com>'
__copyright__ = '2013-2017 Artur Barseghyan'
__license__ = 'GPL 2.0/LGPL 2.1'
__all__ = (
    'MEDIA_ROOT',
    'MEDIA_URL',
    'MEDIA_RELATIVE_ROOT',
    'EXPIRATION_INTERVAL',
    'DEBUG'
)

# Where eximagination cached images will be stored
MEDIA_ROOT = project_dir('media/external_images')
MEDIA_URL = '/media/external_images/'  # Media URL for stored images
MEDIA_RELATIVE_ROOT = 'external_images/'  # Relative root for images
EXPIRATION_INTERVAL = 2592000  # After 30 days we re-fetch the file anyway.
DEBUG = False



import os

__title__ = 'eximagination.helpers'
__author__ = 'Artur Barseghyan <artur.barseghyan@gmail.com>'
__copyright__ = '2013-2017 Artur Barseghyan'
__license__ = 'GPL 2.0/LGPL 2.1'
__all__ = (
    'project_dir',
    'PROJECT_DIR',
)


def project_dir(base):
    """Get path to directory/file from current path."""
    return os.path.join(os.path.dirname(__file__), base).replace('\\', '/')


PROJECT_DIR = project_dir
